import {ProductSystemGenerator} from "./ProductSystemGenerator";
import {ProductGeneratorResult} from "./ProductGeneratorResult";
import {TemplateElement} from "../../templates/TemplateElement";
import {ProductTemplateSystem} from "../../templates/productGamesystem/ProductTemplateSystem";
import {ProductState} from "../states/ProductState";
import {Transition} from "../transitions/Transition";
import {ProductTransition} from "../transitions/ProductTransition";
import {State} from "../states/State";
import {SimpleTemplateTransition} from "../../templates/simpleGamesystem/SimpleTemplateTransition";
import {state, transition} from "@angular/animations";
import {ScriptAccountCondition} from "../conditions/ScriptAccountCondition";
import {SimpleTemplateState} from "../../templates/simpleGamesystem/SimpleTemplateState";

export class TemplateProductSystemGenerator extends ProductSystemGenerator {

  templateElement: TemplateElement


  constructor(productGamesystem: ProductTemplateSystem, templateElement: TemplateElement) {
    super(productGamesystem);
    this.templateElement = templateElement;
  }

  protected assignGeneratedStatesAndTransitions(generationResult: ProductGeneratorResult) {
    const productTemplateSystem = this.productGamesystem as ProductTemplateSystem
    productTemplateSystem.transitionMap.set(this.templateElement, generationResult.transitions)
    productTemplateSystem.stateMap.set(this.templateElement, generationResult.states)
  }

  protected getTransitionConditions(transition: Transition<any>) {
    if(transition instanceof SimpleTemplateTransition) {
      return transition.conditionMap.get(this.templateElement)!
    } else {
      return transition.scriptAccountConditions;
    }
  }

  protected getTransitionActions(transition: Transition<any>) {
    if(transition instanceof SimpleTemplateTransition) {
      return transition.actionMap.get(this.templateElement)!
    } else {
      return transition.scriptAccountActions;
    }
  }


  protected getStateConditions(state: State<any>): ScriptAccountCondition[] {
    if(state instanceof  SimpleTemplateState) {
      return state.conditionMap.get(this.templateElement)!
    } else {
      return state.conditions
    }
  }
}
